
package org.nla.cobol;

import java.util.List;
import java.util.ArrayList;
import org.nla.utils.StringFormater;


/**
 * Generic cobol element
 * @author NLA
 */

/**
 * @author nico
 *
 */
public abstract class CobolElement {

	/////////////////////////////////////////////////////////////////
	///		Members
	/////////////////////////////////////////////////////////////////
	
	/** depth of occurs, meaning how many parent elements are occursed in the hierarchy */
	protected int _intOccursDepth;
	
	/** name of the element */
	protected String _strName;
	
	/** is the element occursed  */
	protected boolean _blnOccurs;
	
	/** number of occurs of this element */
	protected int _intOccursCount;

	/** occurs indices of this element - number of elements in the array must match the occurs depth */
	protected List<Integer> _lstIndices;

	/** level of this element */
	protected int _intLevel;

	/** level of this element, as String */
	protected String _strLevel;
	
	/** start position of this element */
	protected int _intStartPosition;
	
	/** end position of this element */
	protected int _intEndPosition;
	
	/** root copybook of all elements */
	protected CobolCopyBook ourParentToUsAll;
	
	
	
	/////////////////////////////////////////////////////////////////
	///		Constructors
	/////////////////////////////////////////////////////////////////
	
	/**
	 * Default and only constructor
	 */
	protected CobolElement() {
		_lstIndices = new ArrayList<Integer>(); 
		_intOccursCount = 1;
		_intOccursDepth = 0;
	}

	/////////////////////////////////////////////////////////////////
	///		Getters
	/////////////////////////////////////////////////////////////////
	
	/**
	 * Returns the CopyBook to which this element belongs to
	 * @return the parent copybook
	 */
	//@Override
	public CobolCopyBook getOurParentOfUsAll() {
		return ourParentToUsAll;
	}

	/**
	 * Returns the element name
	 * @return	element name
	 */
	//@Override
	public String getName()	{
		return _strName;
	}

	/**
	 * Tells whether the element is occursed
	 * @return	element occurs state 
	 */
	////@Override
	public boolean isOccursed()	{
		return _blnOccurs;
	}

	/**
	 * Returns the element occurs count
	 * @return	element occurs count
	 */
	//@Override
	public int getOccursCount()	{
		return _intOccursCount;
	}
	
	/**
	 * Returns the element level as String
	 * @return	element level as String
	 */
	//@Override
	public String getLevelAsString() {
		return _strLevel;
	}

	/**
	 * Returns element level
	 * @return	element level
	 */
	//@Override
	public int getLevel() {
		return _intLevel;
	}

	/**
	 * Returns element occurs depth
	 * @return	element occurs depth
	 */
	//@Override
	public int getOccursDepth() {
		return _intOccursDepth;
	}

	/**
	 * Returns element start position
	 * @return	element start position
	 */
	//@Override
	public int getStartPosition() {
		return( _intStartPosition );
	}

	/**
	 * Returns element end position
	 * @return	element end position
	 */
	//@Override
	public int getEndPosition() {
		return( _intEndPosition );
	}

	
	
	/////////////////////////////////////////////////////////////////
	///		Setters
	/////////////////////////////////////////////////////////////////

	/**
	 * Sets the CopyBook to which this element belongs to
	 * @param ourParentOfUsAll 	parent copybook to set 
	 */
	//@Override
	public void setOurParentOfUsAll(CobolCopyBook ourParentOfUsAll) {
		this.ourParentToUsAll = ourParentOfUsAll;
	}

	/**
	 * Sets the occurs indices of the element 
	 * @param	lstIndices	indices to set
	 */
	//@Override
	public void setIndices( List<Integer> lstIndices ) {
		_lstIndices = lstIndices;
	}
	
	/**
	 * Sets the name of the element
	 * @param	name	name to set
	 */
	//@Override
	public void setName(String name)	{
		_strName = name;
	}
	
	/**
	 * Sets the occurs state of the element
	 * @param	isOccurs	occurs state to set
	 */
	//@Override
	public void setOccursed(boolean isOccurs)	{
		 _blnOccurs = isOccurs;
		 if ( !_blnOccurs )
			 _intOccursCount = 1;
	}
	
	/**
	 * Sets the number of occurs of the element
	 * @param	occursCount		number of occurs to set
	 */
	//@Override
	public void setOccursCount(int occursCount)	{
		_intOccursCount = occursCount;
	}

	/**
	 * Sets occurs depth of the element 
	 * @param	occursDepth		occurs depth to set
	 */
	//@Override
	public void setOccursDepth(int occursDepth) {
		_intOccursDepth = occursDepth;
	}

	/**
	 * Sets start position of the element 
	 * @param	startPosition		startPosition to set
	 */
	//@Override
	public void setStartPosition(int startPosition) {
		_intStartPosition = startPosition;
	}

	/**
	 * Sets end position of the element 
	 * @param	endPosition		endPosition to set
	 */
	//@Override
	public void setEndPosition(int endPosition) {
		_intEndPosition = endPosition;
	}		
	
	/**
	 * Sets level as a String of element 
	 * @param	level		level as String to set
	 */
	//@Override
	public void setLevelAsString(String level) {
		try {
			setLevel(Integer.parseInt(level));
		}
		catch( Exception e) {
			throw( new IllegalArgumentException( "CobolVariable::setLevelAsString(String) non numeric level " + level + " for variable [" + _strName + "]" ) );
		}
		//_strLevel = level;
	}

	/**
	 * Sets level of element
	 * @param	level		level to set
	 */
	public void setLevel(int level) {
		
		if ( level < 0 ) {
			throw( new IllegalArgumentException( "CobolVariable::setLevel() incorrect level " + level + " for variable [" + _strName + "]" ) );
		}
		_intLevel = level;
		if ( _intLevel < 10 ) {
			_strLevel = "0" + level;
		}
		else {
			_strLevel = "" + level;
		}
	}
	
	/////////////////////////////////////////////////////////////////
	///		Methods
	/////////////////////////////////////////////////////////////////

	
	/**
	 * Returns the name of the element followed by as many question marks as there are indices. 
	 * For example AAA[?,?,?] if variable AAA had an occurs depth of 3. 
	 */
	//@Override
	public String getReferencedName() {
		if ( _intOccursDepth > 0 ) {
			return ( _strName + buildReferences(_intOccursDepth));
		}
		return _strName;
	}
	
	/**
	 * Returns the name of the element followed its indices.  
	 * For example AAA[1,3,5] if variable AAA had an occurs depth of 3 and the indices {1, 3, 5} 
	 */
	//@Override
	public String getAbsoluteName() {
		String s = StringFormater.formatSequenced(getReferencedName(), _lstIndices.toArray()); 
		return s;
	}
	
	
	/**
	 * @param	nbClones	number of clones to create
	 * @param	startIndex	index out of nbClones from where to start creating clones
	 *  @param	lstIndicesFromParent	indices of parent element
	 */
	public List<CobolElement> cloneItself( int nbClones, int startIndex, List<Integer> lstIndicesFromParent ) {
		ArrayList<CobolElement> lstClones = new ArrayList<CobolElement>();
		
		for ( int i = startIndex; i <= nbClones; ++i ) {
			CobolElement ce = cloneItself();
			List<Integer> clonedLstIndices = cloneList(lstIndicesFromParent);
			clonedLstIndices.add(i);
			ce.setIndices(clonedLstIndices);
			lstClones.add( ce );
		}
		
		return ( lstClones );
	}

	/////////////////////////////////////////////////////////////////
	///		Abstract methods, to be overriden by sub-classes
	/////////////////////////////////////////////////////////////////
	
	/**
	 * Returns its final type
	 */
	public abstract String getFinalType();

	/**
	 * Builds and returns a clone of itself 
	 */
	public abstract CobolElement cloneItself();

	/**
	 * Returns its size 
	 */
	public abstract int getSize();

	/**
	 * Returns a count of its elements
	 */
	public abstract int getCount();
	
	
	/////////////////////////////////////////////////////////////////
	///		Static Methods
	/////////////////////////////////////////////////////////////////
	
	/**
	 * Clones and returns a list of Integers. 
	 * @param 	toClone		List of Integers to clone
	 * @return	cloned list
	 */
	protected static List<Integer> cloneList(List<Integer> toClone) {
		ArrayList<Integer> clone = new ArrayList<Integer>( toClone.size() );
		
		for ( Integer ii : toClone ) {
			clone.add( ii.intValue() );
		}
		
		return clone;
	}
	
	/**
	 * Builds a String consisting of <i>occursDepth</i> question marks. 
	 * For instance, if <i>occursDepth</i> = 3, this method will return "[?,?,?]" 
	 * @param 	occursDepth		Number of question marks to add in the String.
	 * @return	a String containing <i>occursDepth</i> questions marks
	 */
	protected static String buildReferences(int occursDepth) {
		StringBuilder s = new StringBuilder();
		s.append("[?");
		
		for ( int i = 1; i < occursDepth; ++i ) {
			s.append(",?");
		}
		
		s.append("]");
		
		return s.toString();
	}

	/**
	 * Builds a String consisting of <i>nbLevels</i> double spaces. 
	 * @param 	nbLevels		Number of double space to concat in the String.
	 * @return	a String containing <i>nbLevels</i> double spaces.
	 */
	protected static String buildLevels(int nbLevels) {
		StringBuilder s = new StringBuilder();
				
		for ( int i = 1; i < nbLevels; ++i ) {
			s.append("  ");
		}
		
		return s.toString();
	}
	


	/////////////////////////////////////////////////////////////////
	///		Test Methods
	/////////////////////////////////////////////////////////////////

	public static void main(String args[]) {
		CobolElement cv = new CobolVariable();
		cv.setName( "test" );
		cv.setOccursDepth(3);
		ArrayList<Integer> lst = new ArrayList<Integer>();
		lst.add(1);
		lst.add(3);
		lst.add(2);
		cv.setIndices(lst);
		
		System.out.println( cv.getName() );
		System.out.println( cv.getReferencedName() );
		System.out.println( cv.getAbsoluteName() );
	}
}
