
package org.nla.cobol;

/**
 * Representation of a COBOL Field 
 * @author NLA
 */
public class CobolVariable extends CobolElement
{

	/////////////////////////////////////////////////////////////////
	///		Constants
	/////////////////////////////////////////////////////////////////
	
	/** filler keyword */
	private static final String STR_FILLER = "FILLER";
	
	/** underscore */
	private static final String STR_US = "_";
	
	/** S for Signed */
	private static final String STR_SIGNED = "S";

	/** Left bracket */
	private static final String STR_LBRACKET = "(";

	/** Right bracket */
	private static final String STR_RBRACKET = ")";
	
	/** V for Virgule ??? */
	private static final String STR_VIRGULE = "V";
	
	/** Binary keyword */
	private static final String STR_BINARY = "BINARY";
	
	

	/////////////////////////////////////////////////////////////////
	///		Members
	/////////////////////////////////////////////////////////////////
	
	/** cobol type of field : X or 9 */
	private CobolType _cblType;
	
	/** integer size of numeric field */
	private int _intIntSize;
	
	/** decimal size of numeric field */
	private int _intDecSize;
	
	/** is numeric field decimal */
	private boolean _blnDecimal;
	
	/** is numeric field signed */
	private boolean _blnSigned;
	
	/** is field binary/comp-3 */
	private boolean _blnComp3;


	/////////////////////////////////////////////////////////////////
	///		Getters
	/////////////////////////////////////////////////////////////////
	
	/**
	 * Returns inner cobol type of the field : X (String) or 9 (Numeric)
	 * @return	cobol type of the field 
	 */
	public CobolType getCobolType()	{
		return _cblType;
	}

	/**
	 * Returns integer size of the field. This method is not dependant on the type.
	 * @return	 integer size of the field
	 */
	public int getIntSize()	{
		return _intIntSize;
	}	
	
	/**
	 * Returns decimal size of the field. This method returns 0 if the type of the field is not numeric.
	 * @return	 decimal size of the field, zero if its type is not numeric 
	 */
	public int getDecSize()	{
		return ( _cblType == CobolType.NUM ? _intDecSize : 0);
	}	

	/**
	 * Returns true if the type of the field is decimal 
	 * @return	 true if the field is decimal, false otherwise
	 */
	public boolean isDecimal()	{
		return ( _cblType == CobolType.NUM ? _blnDecimal : false);
	}	

	/**
	 * Returns true if the field is signed.
	 * @return	 true if the field is signed, false otherwise.
	 */
	public boolean isSigned()	{
		return ( _cblType == CobolType.NUM ? _blnSigned : false);
	}

	/**
	 * Returns true if the field is declared binary or comp-3.
	 * @return	 true if the field is declared binary or comp-3
	 */
	public boolean isComp3() {
		return _blnComp3;
	}

	/////////////////////////////////////////////////////////////////
	///		Setters
	/////////////////////////////////////////////////////////////////

	/**
	 * Sets cobol type of the field : X (String) or 9 (Numeric)
	 * @param	cblType	cobol type to set 
	 */
	public void setCobolType(CobolType cblType)	{
		_cblType = cblType;
	}

	/**
	 * Sets integer size of the field
	 * @param	intSize		integer size to set 
	 */
	public void setIntSize(int intSize)	{
		_intIntSize = intSize;
	}	

	/**
	 * Sets decimal size of the field
	 * @param	decSize		decimal size to set 
	 */
	public void setDecSize(int decSize)	{
		_intDecSize = decSize;
	}	

	/**
	 * Sets decimal state of the field 
	 * @param	isDecimal	boolean indicating whether the field is decimal 
	 */
	public void setDecimal(boolean isDecimal)	{
		_blnDecimal = isDecimal;
	}	

	/**
	 * Sets signed state of the field 
	 * @param	isSigned	boolean indicating whether the field is signed
	 */
	public void setSigned(boolean isSigned)	{
		_blnSigned = isSigned;
	}
	
	/**
	 * Sets binary state of the field 
	 * @param	isComp3		boolean indicating whether the field is binary/comp-3
	 */
	public void setComp3(boolean isComp3)	{
		 _blnComp3 = isComp3;
	}
	
	/////////////////////////////////////////////////////////////////
	///		Constructors
	/////////////////////////////////////////////////////////////////


	/** 
	 *	Default and only constructor of a cobol field.
	 */
	public CobolVariable() {
		super();
		_blnComp3 = false;
		_blnSigned = false;
	}


	/////////////////////////////////////////////////////////////////
	///		Methods
	/////////////////////////////////////////////////////////////////

	/**
	 * Sets the name of the cobol field. To prevent the fact there can be several fields named FILLER and then have <br>
	 * problems in the copybook, this method adds up an index at the end of the name if this name equals "FILLER"  
	 * @param	strName		name to set 
	 */
	@Override
	public void setName(String strName)	{
		if ( strName.equalsIgnoreCase( STR_FILLER ) ) {
			_strName = strName + STR_US + CobolCopyBook.intFillerCount;
			CobolCopyBook.intFillerCount += 1;
		}
		else {
			_strName = strName;
		}
	}
	
	/**
	 * Returns the final type of the cobol field. The pattern will match <i>[S]T(IS)[V9(DS)] [BINARY]</i> where : <br>
	 * &nbsp;&nbsp; - S is present if the field is signed <br>
	 * &nbsp;&nbsp; - T is either X or 9 <br>
	 * &nbsp;&nbsp; - IS is the integer size of the field<br>
	 * &nbsp;&nbsp; - V9(DS) is present if the field is decimal 
	 * &nbsp;&nbsp; - DS is the decimal size of the field<br>
	 * &nbsp;&nbsp; - BINARY is present if the field is declared binary or comp-3<br>
	 * note that this pattern will be returned whatever the way the type was declared in the copybook. For instance, 
	 * if the field A was declared as <i>S9(2)V999 comp-3</i> this method will return <i>S9(2)V9(3) BINARY</i>  
	 * @return	final type of the cobol field following the pattern <i>[S]T(IS)[V9(DS)] [BINARY]</i>
	 */
	@Override
	public String getFinalType() {
		
		StringBuilder type = new StringBuilder();
		
		// if field is signed, start with a S
		if ( _blnSigned ) {	
			type.append( STR_SIGNED );
		}
		// this part is mandatory : T(IS)
		type.append( _cblType.getInnerType() );
		type.append( STR_LBRACKET );
		type.append( _intIntSize );
		type.append( STR_RBRACKET );
		
		// this part is present if decimal : 9(DS)
		if ( _blnDecimal ) {
			type.append( STR_VIRGULE );
			type.append( STR_LBRACKET );
			type.append( _intDecSize );
			type.append( STR_RBRACKET );
		}
		
		// if field is binary, finish with BINARY
		if ( _blnComp3 ) {
			type.append( STR_BINARY );
		}
		
		return type.toString();
	}
	
	
	/**
	 * Returns a sum of: integer size + decimal size + 1 if field is signed
	 * @returns 	 sum of integer size + decimal size + 1 if field is signed
	 */
	private int getBaseSize() {
		return _intIntSize + _intDecSize + ( _blnSigned ? 1 : 0 );
	}

	
	/**
	 * Returns the actual size of by this field. It corresponds to : <br>
	 * &nbsp;&nbsp;		- the base size of this field added to 1, divided by 2 and rounded up if the field is binary <br> 
	 * &nbsp;&nbsp;		- the base size or the previous computations multiplied by the number of occurs of this field     
	 * @returns 	 actual size of the field
	 */
	@Override
	public int getSize() {
		int intBaseSize = getBaseSize();
		
		if ( _blnComp3 ) {
			double dblSize = (double)intBaseSize;
			dblSize = ( intBaseSize + 1.00 ) / 2.00;
			intBaseSize = (int)Math.ceil( dblSize ); 
		}
		return ( intBaseSize ) * _intOccursCount;
	}
	
	/**
	 * Returns the number of fields designed by this element. In case of a field, it corresponds to the number of 
	 * its occurs count. 
	 * @returns 	 number of occurs count 
	 */
	@Override
	public int getCount() {
		return getOccursCount();
	}
	
	/**
	 * Returns a new CobolVariable cloned with this field's properties.
	 * @returns 	a clone of this  
	 */
	@Override
	public CobolElement cloneItself() {
		CobolVariable cv = new CobolVariable();
		cv.setOurParentOfUsAll( ourParentToUsAll );
		cv.setLevel(_intLevel);
		cv.setName(_strName);
		cv.setCobolType(_cblType);
		cv.setDecimal(_blnDecimal);
		cv.setDecSize(_intDecSize);
		cv.setIntSize(_intIntSize);
		cv.setOccursed(_blnOccurs);
		cv.setOccursCount(_intOccursCount);
		cv.setOccursDepth(_intOccursDepth);
		cv.setOurParentOfUsAll( ourParentToUsAll );
		return ( cv );
	}

	/**
	 * Returns a String representation of this field
	 * @returns 	 number of occurs count 
	 */
	@Override
	public String toString() {
		return ( buildLevels(_intLevel) + _strLevel + " " + getAbsoluteName() + " " + _cblType + " " + getBaseSize() + ( _blnComp3 ? "B(" + getSize() + ")" : "" ) + " " + (_blnOccurs ? "OCCURS x " + _intOccursCount : "" ) + "(" + _intStartPosition + "," + _intEndPosition + ")" + "\n" );
	}

	/**
	 * Returns true if the specified object is of type CobolVariable and its name is equals to this name.
	 * @param		o	an object
	 * @returns 	true if o is a CobolVariable and its name is equal to this name  
	 */
	@Override
	public boolean equals( Object o )
	{
		if (o instanceof CobolVariable) {
			CobolElement param = (CobolElement)o;
			if ( param.getName().equals( _strName ) ) {
				return true;
			}
		}
		
		return false;
	}
	
	/////////////////////////////////////////////////////////////////
	///		Test Methods
	/////////////////////////////////////////////////////////////////
	
	public static void main(String[] args)
	{
		CobolVariable v = new CobolVariable();
		System.out.println(v);
		
		v.setLevelAsString("05");
		v.setName("LETSASRP-INDBAS");
		v.setIntSize(5);
		v.setCobolType(CobolType.NUM);
		
		System.out.println(v);
	}
}