
package org.nla.cobol;


import java.util.Iterator;

/**
 * Mapper between a fields of copybook and lines of a file   
 * @author NLA
 */
public class CopyFileMapper {

	/////////////////////////////////////////////////////////////////
	///		Members
	/////////////////////////////////////////////////////////////////
	
	/** file containing the lines to map to the copybook fields*/
	private CobolFile cobolFile;
	
	/** copybook defining the fields to map to the file lines */
	private CobolCopyBook cobolCopyBook;
	

	/////////////////////////////////////////////////////////////////
	///		Constructors
	/////////////////////////////////////////////////////////////////
	
	/**
	 * Costructs a mapper using a cobol file and a copybook
	 * @param	cobolFile		CobolFile containing the lines to map
	 * @param	cobolCopyBook	CopyBook defining the fields to use
	 */
	public CopyFileMapper(CobolFile cobolFile, CobolCopyBook cobolCopyBook) {
		super();
		this.cobolFile = cobolFile;
		this.cobolCopyBook = cobolCopyBook;
	}

	
	/////////////////////////////////////////////////////////////////
	///		Methods 
	/////////////////////////////////////////////////////////////////
	
	/**
	 * Map a file line to the copybook fields, ans returns a CobolCopyBookLineMapping.
	 * @param		lineNumber	number of file line to map to the copybook fields
	 * @returns		a CobolCopyBookLineMapping containing a mapping between the given line and the copybook fields
	 */
	public CobolCopyBookLineMapping mapFileLineToCopyBook( int lineNumber ) {
		
		CobolCopyBookLineMapping toReturn = new CobolCopyBookLineMapping();
		
		cobolFile.setCurrentPosition( lineNumber );
		String cobolFileLine = cobolFile.getCurrentLine();

		Iterator<String> keyIterator = cobolCopyBook.keyIterator();
		while ( keyIterator.hasNext() ) {
			String elementName = keyIterator.next();
			CobolElementValueMapping elementValueMapping = new CobolElementValueMapping();
			CobolElement element = cobolCopyBook.getCobolElement( elementName );
	
			String elementValue = null;
			if ( element.getEndPosition() > cobolFileLine.length() ) {
				if ( element.getStartPosition() > cobolFileLine.length() ) {
					elementValue = "";
				}
				else {
					elementValue = cobolFileLine.substring( element.getStartPosition() - 1, cobolFileLine.length() );
				}
			}
			else {
				elementValue = cobolFileLine.substring( element.getStartPosition() - 1, element.getEndPosition() );
			}
			elementValueMapping.setCobolElementName( elementName );
			elementValueMapping.setElement( element );
			elementValueMapping.setElementValue( elementValue );
			
			toReturn.addMapping( elementValueMapping );
		}
		
		return toReturn;
	}





	/////////////////////////////////////////////////////////////////
	///		Test methods 
	/////////////////////////////////////////////////////////////////
	
	public static void main(String[] args) {
		try {
			CobolFile file = new CobolFile("extract.txt", true);
			CobolCopyBook copy = CobolCopyBookParser.buildCopyBook("copy.txt");
			
			
			CopyFileMapper mapper = new CopyFileMapper( file, copy );
			
			CobolCopyBookLineMapping ccblm = mapper.mapFileLineToCopyBook( 1 );
			System.out.println( ccblm.getMappingSize() );
			
			
			
			for ( int i = 0; i < ccblm.getMappingSize(); ++i )
			{
				CobolElementValueMapping cevm = ccblm.getMapping(i);
				System.out.println(cevm.getCobolElementName() + " :'" + cevm.getElementValue()+"'");
			}
			
			

		}
		catch ( Exception e ) {
			e.printStackTrace();
		}
		

	}

}
