
////////////////////////////////////////////////////////////////////
// Package
////////////////////////////////////////////////////////////////////

package org.nla.utils;

////////////////////////////////////////////////////////////////////
// Imports
////////////////////////////////////////////////////////////////////

import java.util.Map;


////////////////////////////////////////////////////////////////////
// Class definition
////////////////////////////////////////////////////////////////////

public class StringFormater
{
   ////////////////////////////////////////////////////////////////////
   // Constants
   ////////////////////////////////////////////////////////////////////

   /** at sign character */
   private static final char AT_SIGN_CHAR = '@';

   /** open { */
   private static final char OPEN_BRACKET = '{';

   /** closing } */
   private static final char CLOSING_BRACKET = '}';

   /** backslash character */
   private static final char BACKSLASH_CHAR = '\\';

   /** question mark */
   private static final char QUESTION_MARK = '?';


   
   ////////////////////////////////////////////////////////////////////
   // Static methods
   ////////////////////////////////////////////////////////////////////

   
   /**
    * Reformats String by replacing its parameters by specified values. Its parameters are identified by &#64 signs.
    * The parameters in the input String are replaced by the named values in the parameter Map
    * 
    * Example: <br>
    * <i>String to format:</i> 	"insert into table &#64table(&#64columns) values(&#64values)"<br>
    * <i>Parameter Map:</i> 	{ "table"="person_t" ; "columns"="name, age" ; "values"="'john', 25" }<br>
    * <i>Returned String:</i> 	"insert into table personne_t(name, age) values('john', 25)"<br>
    * 
    * @param strToFormat	String to format, containing named parameters identified by &#64 signs
    * @param mapParameters	Map containing a parameter value for each parameter name
    * @return	String whose &#64 parameters are replaced by specified values in the Map
    */
   public static String formatAtSigned( String strToFormat, Map mapParameters )
   {
	  // checks whether input string contains parameters
	  if ( !strToFormat.contains( "@" ) )
	  {
		  return ( strToFormat );
	  }
	   
      StringBuffer strToReturn = new StringBuffer();
      boolean indexInParameter = false;

      char[] charactersToFormat = strToFormat.toCharArray();
      for ( int i = 0; i < charactersToFormat.length && charactersToFormat.length > 1; ++i )
      {
         char currentChar = charactersToFormat[i];

         // at sign at the beginning OR at sign after a non letter or digit character
         // search param name and look for it in param table
         if (
               ( currentChar == AT_SIGN_CHAR && i == 0 ) ||
               ( currentChar == AT_SIGN_CHAR && charactersToFormat[i-1] != BACKSLASH_CHAR )
            )
         {
            indexInParameter = true;
            // at signed parameter name construction
            StringBuffer parameterName = new StringBuffer();
            for ( int j = i + 1; j < charactersToFormat.length; ++j )
            {
               currentChar = charactersToFormat[j];
               if ( Character.isLetterOrDigit( currentChar ) || currentChar == '_' )
               {
                  parameterName.append( currentChar );
               }
               else
               {
                  indexInParameter = false;
                  i = j;
                  break;
               }
               i = j;
            }
            if ( i == charactersToFormat.length )
            {
               indexInParameter = true;
            }

            // parameter value replacement if exists
            Object parameterValue = mapParameters.get( parameterName.toString() );
            if ( parameterValue != null )
            {
               strToReturn.append( parameterValue.toString() );
               //System.out.println( i+";"+charactersToFormat.length );
               if ( !indexInParameter ) strToReturn.append( currentChar );
            }
            else
            {
               strToReturn.append( AT_SIGN_CHAR );
               strToReturn.append( parameterName );
            }
         }
         else
         {
            strToReturn.append( currentChar );
         }
      }

      return ( strToReturn.toString() );
   }

   
   /**
    * Reformats String by replacing its parameters by specified values. 
    * Its parameters are identified by {x} character sequences, where x is the parameter index.  
    * The parameters in the input String are replaced by their corresponding values in the array. 
    * Each {x} parameter is replaced by the value of x position in the array
    *    
    * Example: <br>
    * 	<i>String to format:</i> 	"select {0} from {1} a, {1} b where a.{2} <= b.{5}"
    * 	<i>Parameters:</i> 			{ "name", "personne_t", "personne_id", "age", "unused", "age" }
    * 	<i>Returned String:</i> 	"select name from personne_t a, personne_t b where a.age = b.age"
    *   
    * @param strToFormat	String to format, containing parameters identified by {x} character sequences
    * @param parameters		Array containing values with which to replace the input String parameters 
    * @return	String whose {x} parameters are replaced by specified values in the array  
    */
   public static String formatIndexed( String strToFormat, Object[] parameters )
   {
	  // checks whether input string contains parameters
	  if ( !strToFormat.contains( "{" ) )
	  {
		  return ( strToFormat );
	  }
	   
      StringBuffer strToReturn = new StringBuffer();

      char[] charactersToFormat = strToFormat.toCharArray();
      for ( int i = 0; i < charactersToFormat.length && charactersToFormat.length > 3; ++i )
      {
         char currentChar = charactersToFormat[i];

         // open bracket at the beginning or not after a backslash
         // replace it with the number right after
         if (
               (
                  ( currentChar == OPEN_BRACKET && i == 0 ) ||
                  ( currentChar == OPEN_BRACKET && charactersToFormat[i-1] != BACKSLASH_CHAR )
               )
               && Character.isDigit( charactersToFormat[i+1] )
               && charactersToFormat[i+2] == CLOSING_BRACKET
            )
         {
            int paramIndex = Integer.parseInt( new Character( charactersToFormat[i+1] ).toString() );
            try
            {
               strToReturn.append( parameters[paramIndex] );
            }
            // parameter index out of bounds
            catch( IndexOutOfBoundsException ioobe )
            {
               strToReturn.append( OPEN_BRACKET );
               strToReturn.append( charactersToFormat[i+1] );
               strToReturn.append( CLOSING_BRACKET );
            }
            i += 2;
         }

         // normal character, append it
         else
         {
            strToReturn.append( currentChar );
         }
      }
      return strToReturn.toString();
   }


   /**
    * Reformats String by replacing its parameters by specified values. Its parameters are identified by ? characters. 
    * The parameters in the input String are replaced by the values in the array, following the order of the array. 
    * Example: <br>
    * 	<i>String to format:</i> 	"select ?,? from ? where ? = ?"<br>
    * 	<i>Parameters:</i> 			{ "name", "age", "personne_t", "personne_id", "5" }
    * 	<i>Returned String:</i> 	select name,age from personne_t where personne_id = 5
    *   
    * @param strToFormat	String to format, containing parameters identified by ? characters
    * @param parameters		Array containing values with which to replace the input String parameters 
    * @return	String whose ? parameters are replaced by specified values in the array  
    */
   public static String formatSequenced( String strToFormat, Object[] parameters )
   {

	  // checks whether input string contains parameters
	  if ( !strToFormat.contains( ""+QUESTION_MARK ) )
	  {
		  return ( strToFormat );
	  }
	   
      StringBuffer toReturn = new StringBuffer();

      char[] charactersToFormat = strToFormat.toCharArray();

      for ( int i = 0, paramIndex = 0; i < charactersToFormat.length; ++i )
      {
         char currentChar = charactersToFormat[i];

         // question mark, replace it with its index in parameters table
         if ( currentChar == QUESTION_MARK && paramIndex < parameters.length )
         {
            toReturn.append( parameters[paramIndex] );
            paramIndex += 1;
         }

         // normal character, append it
         else
         {
            toReturn.append( currentChar );
         }
      }

      return toReturn.toString();
   }
   

   ////////////////////////////////////////////////////////////////////
   // Constructors
   ////////////////////////////////////////////////////////////////////
   
   // not instantiable
   private StringFormater()
   {
   }


   ////////////////////////////////////////////////////////////////////
   // Unit testing
   ////////////////////////////////////////////////////////////////////


   /**
    * Unit testing
    */
   public static void main( String[] args )
   {
	   Debug.log("coucou");
		//
		// format @ parameters
		//

      java.util.Map parameters = new java.util.HashMap();
      parameters.put( "table", "personne_t" );
      parameters.put( "columns", "nom, prenom, age" );
      parameters.put( "values", "'laurent', 'nicolas', 25" );

      String rawSql = "insert into table @table(@columns) values(@values)";
      String sql = formatAtSigned( rawSql, parameters );
      System.out.println( "sql 1 : " + sql );

		//
		// format sequenced parameters
		//

		rawSql = "select ?,?,? from ? where ? = ?";
		String[] args1 = { "nom", "prenom", "age", "personne_t", "personne_id", "5" };
		sql = formatSequenced( rawSql, args1 );
      System.out.println( "sql 2 : " + sql );

		//
		// format sequenced parameters
		//

		rawSql = "select {0} from {1} a, {1} b where a.{2} <= b.{5}";
		String[] args2 = { "*", "personne_t", "age", "age2", "age3", "age4" };
		sql = formatIndexed( rawSql, args2 );
      System.out.println( "sql 3 : " + sql );

   }
}